##############################################################################
# Module : dhtrerequester.py
# Author : Old King Cole, Bram Cohen
# Date   : 11 February 2009
#
# Description : DHT rerequester
#
##############################################################################
from random import shuffle
from time import clock

from BitTornado.utility import exceptionArgsToString, getABCUtility


class DHTRerequester:
    def __init__(self, infohash, config, dht, sched, errorfunc,
                 connect, howmany, unpauseflag, listenport):
        self.infohash = infohash
        self.dht = dht
        self.sched = sched
        self.errorfunc = errorfunc
        self.connect = connect
        self.howmany = howmany
        self.unpauseflag = unpauseflag
        self.listenport = listenport
        self.last_announce = 0
        self.stopped = True
        self.minpeers = config['min_peers']
        self.interval = config['dhtrerequest_interval']
        self.announce_interval = 1800
        # 0 : not announcing ; 1 : auto announcing ; 2 : manual announcing
        self.announcing = 0

    def start(self):
        self.stopped = False
        self.d()

    def d(self):
        if self.stopped:
            return
        if not self.unpauseflag.isSet():
            self.stopped = True
            return
        if not self.announcing and (self.howmany() < self.minpeers or clock() - self.last_announce > self.announce_interval):
            self.rerequest(False)
        else:
            self._d()

    def _d(self):
        self.sched(self.d, self.interval)

    def rerequest(self, manual):
        if self.announcing:
            return
        if manual:
            self.announcing = 2
        else:
            self.announcing = 1
        self.last_announce = clock()
        self.dht.sched(self.dht.valueForKeyAndStore, 0, [self.infohash, self.listenport, self.gotPeers])

    def gotPeers(self, peerinfos):
        self.sched(self._gotPeers, 0, [peerinfos])

    def _gotPeers(self, peerinfos):
        if not self.stopped and self.unpauseflag.isSet():
            # Last call to gotPeers for a rerequest is performed with empty peerinfos
            if peerinfos:
                #print 'GOT DHT PEERS', peerinfos
                shuffle(peerinfos)
                self.connect([(peerinfo, 0, None, None, 2) for peerinfo in peerinfos])
            else:
                if self.announcing == 1:
                    self._d()
                self.announcing = 0
